﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.IO;
using System.Windows.Media.Imaging;
using System.Windows.Markup;
using System.Threading;
using System.Globalization;

namespace OpenFileDialogDemo
{
    public partial class Page : UserControl
    {
        private const int _cx = 400;
        private const int _cy = 300;
        private const int _border = 10;
        private const int _angle = 30;
        private bool _dragging = false;
        private double _lastx, _lasty;
        private FrameworkElement _owner;
        private Random _rand = new Random(DateTime.Now.Millisecond);
        private int _zindex = 100;
        private int _count = 0;
        private Queue<FileInfo> _files = new Queue<FileInfo>();

        private const string _template =
            "<Border Name=\"Photo_{0}\" Canvas.Left=\"{1}\" Canvas.Top=\"{2}\" Width=\"{3}\" Height=\"{4}\" Background=\"White\" Visibility=\"Collapsed\" xmlns=\"http://schemas.microsoft.com/client/2007\">" +
            "<Border.RenderTransform>" +
            "<RotateTransform Name=\"SpinTransform\" CenterX=\"{5}\" CenterY=\"{6}\" />" +
            "</Border.RenderTransform>" +
            "<Image Width=\"{7}\" Height=\"{8}\" Stretch=\"UniformToFill\" /> " +
            "</Border>";

        public Page()
        {
            InitializeComponent();

            // Center the Open button horizontally
            double width = Application.Current.Host.Content.ActualWidth;
            OpenButton.SetValue(Canvas.LeftProperty, (width - OpenButton.Width) / 2.0);

            // Register a handler for the control's Resized events
            Application.Current.Host.Content.Resized += new EventHandler(OnResized);

            // Register a handler for Rendering events
            CompositionTarget.Rendering += new EventHandler(CompositionTarget_Rendering);
        }

        private void OpenButton_Click(object sender, RoutedEventArgs e)
        {
            // Let the user browse for images
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.Filter = "JPEG Files (*.jpg;*.jpeg)|*.jpg;*.jpeg|PNG Files (*.png)|*.png|All Files (*.*)|*.*";
            ofd.FilterIndex = 1;
            ofd.Multiselect = true;

            if ((bool)ofd.ShowDialog())
            {
                // Measure the width and height of the page and set the PhotoBoard canvas accordingly
                PhotoBoard.Width = Application.Current.Host.Content.ActualWidth;
                PhotoBoard.Height = Application.Current.Host.Content.ActualHeight;

                // Reset the PhotoBoard canvas and photo queue
                PhotoBoard.Children.Clear();
                _files.Clear();
                _count = 0;
                
                // Place each FileInfo in a queue for processing in response to Rendering events
                foreach (FileInfo fi in ofd.Files)
                {
                    _files.Enqueue(fi);
                }
            }
        }

        private void CompositionTarget_Rendering(Object sender, EventArgs e)
        {
            if (_files.Count != 0)
            {
                // Create a photo
                FileInfo fi = _files.Dequeue();
                CreatePhoto(fi);
            }
        }
        
        private void CreatePhoto(FileInfo fi)
        {
            using (Stream stream = fi.OpenRead())
            {
                BitmapImage bi = new BitmapImage();
                bi.SetSource(stream);

                // Create a Border containing an image
                Border item = (Border)XamlReader.Load(String.Format(CultureInfo.InvariantCulture,
                    _template,
                    _count++,
                    (PhotoBoard.Width - _cx) / 2, (PhotoBoard.Height - _cy) / 2, // Initial position (center of workspace)
                    _cx + _border, _cy + _border,                                // Width and height of photo border
                    (_cx + _border) / 2, (_cy + _border) / 2,                    // Center of rotation
                    _cx, _cy));                                                  // Width and height of photo

                // Assign the image bits to the Image
                Image image = (Image)item.Child;
                image.Source = bi;

                // Add the photo to the scene
                PhotoBoard.Children.Add(item);

                // Register event handlers
                item.MouseLeftButtonDown += new MouseButtonEventHandler(OnButtonDown);
                item.MouseMove += new MouseEventHandler(OnMouseMove);
                item.MouseLeftButtonUp += new MouseButtonEventHandler(OnButtonUp);

                // Compute spatial parameters
                double x = _rand.NextDouble() * (PhotoBoard.Width - _cx);
                double y = _rand.NextDouble() * (PhotoBoard.Height - _cy);
                double angle = _angle - (_rand.NextDouble() * _angle * 2);

                // Position the photo and make it visible
                item.SetValue(Canvas.LeftProperty, x);
                item.SetValue(Canvas.TopProperty, y);
                RotateTransform transform = (RotateTransform)item.FindName("SpinTransform");
                transform.Angle = angle;
                item.Visibility = Visibility.Visible;
            }
        }
        
        private void OnButtonDown(object sender, MouseButtonEventArgs e)
        {
            // Move the photo to the top of the Z-order
            ((FrameworkElement)sender).SetValue(Canvas.ZIndexProperty, _zindex++);

            // Record the current mouse position
            _lastx = e.GetPosition(null).X;
            _lasty = e.GetPosition(null).Y;

            // Begin dragging
            _owner = (FrameworkElement)sender;
            ((FrameworkElement)sender).CaptureMouse();
            _dragging = true;
        }

        private void OnMouseMove(object sender, MouseEventArgs e)
        {
            if (_dragging)
            {
                double x = e.GetPosition(null).X;
                double y = e.GetPosition(null).Y;

                double dx = x - _lastx;
                double dy = y - _lasty;

                _lastx = x;
                _lasty = y;

                ((FrameworkElement)sender).SetValue(Canvas.LeftProperty, (double)((FrameworkElement)sender).GetValue(Canvas.LeftProperty) + dx);
                ((FrameworkElement)sender).SetValue(Canvas.TopProperty, (double)((FrameworkElement)sender).GetValue(Canvas.TopProperty) + dy);
            }
        }

        private void OnButtonUp(object sender, MouseButtonEventArgs e)
        {
            ((FrameworkElement)sender).ReleaseMouseCapture();
            _dragging = false;
        }

        private void OnResized(Object sender, EventArgs e)
        {
            // When the control resizes, center the Open button horizontally
            double width = Application.Current.Host.Content.ActualWidth;
            OpenButton.SetValue(Canvas.LeftProperty, (width - OpenButton.Width) / 2.0);
        }
    }
}
